/*-----------------------------------------------------------------------
		
	Inline Math library : Real to Integer conversion by Sree
	
	* See imath.h for more information

	By Matt Barnett
		(see References in imath.h)
	
------------------------------------------------------------------------*/

#ifndef _IMATH_REAL2INT_IMP_HEADER_156858806791
#define _IMATH_REAL2INT_IMP_HEADER_156858806791

// Float/Double -> Int
//
// By Sree
//
//  (from web article)
//	Sree's Real2Int
//
//	There are quite a few ways to fix this. One of the best general conversion utilities I have is one
//	that Sree Kotay wrote. It's only safe to use when your FPU is in double-precision mode, so you have
//	to be careful about it if you're switching precision like above. It'll basically return '0' all the
//	time if you're in single precision.
//
//	--- notes by matt ---
//  Somthing is wierd here
//  My lut class attempts to switch to single precision mode
//  but this function must still be working, or (i think) the
//  core im_ functions would all fall down.
//
//	(I've recently discovered that this discussion bears a lot of resemblance to Chris Hecker's article
//	on the topic, so be sure to read that if you can't figure out what I'm saying.)
//	This function is basically an ANSI C compliant conversion -- it always chops, always does the right thing,
//	for positive or negative inputs.
//
//

//-- performace notes by matt --
//
// the blurb states that (int)somefloat casts
// are very slow.  My lookup table class
// didnt seem to show that(the cast was no slower that this function),
// but i have left this function in, and the class uses it.
// (i guess it may be noticable for doubles (which i am avoiding)
// flipcode threads have also mentioned (somewhere) float->int 
// conversions as being slow.
//

typedef double lreal;
typedef float  real;
typedef unsigned long uint32;
typedef long int32;

const lreal _double2fixmagic = 68719476736.0*1.5;     //2^36 * 1.5,  (52-_shiftamt=36) uses limited precisicion to floor
const int32 _shiftamt        = 16;                    //16.16 fixed point representation,

#if BigEndian_
	#define iexp_				0
	#define iman_				1
#else
	#define iexp_				1
	#define iman_				0
#endif //BigEndian_

//--------------------
// im_real2int(double)
//--------------------
inline int32 im_real2int(lreal val)
{
#if DEFAULT_CONVERSION
	return val;
#else
	val		= val + _double2fixmagic;
	return ((int32*)&val)[iman_] >> _shiftamt; 
#endif
}

//-------------------
// im_real2int(float)
//-------------------
inline int32 im_real2int(real val)
{
#if DEFAULT_CONVERSION
	return val;
#else
	return im_real2int ((lreal)val);
#endif
}


#endif